@@ -1,10 +1,12 @@
 /*
- * parserInternals.h : internals routines exported by the parser.
+ * Summary: internals routines exported by the parser.
+ * Description: this module exports a number of internal parsing routines
+ *              they are not really all intended for applications but
+ *              can prove useful doing low level processing.
  *
- * See Copyright for the status of this software.
- *
- * daniel@veillard.com
+ * Copy: See Copyright for the status of this software.
  *
+ * Author: Daniel Veillard
  */
 
 #ifndef __XML_PARSER_INTERNALS_H__
@@ -259,6 +261,9 @@ XMLPUBFUN int XMLCALL                   xmlIsLetter     (int c);
  */
 XMLPUBFUN xmlParserCtxtPtr XMLCALL	
 			xmlCreateFileParserCtxt	(const char *filename);
+XMLPUBFUN xmlParserCtxtPtr XMLCALL	
+			xmlCreateURLParserCtxt	(const char *filename,
+						 int options);
 XMLPUBFUN xmlParserCtxtPtr XMLCALL	
 			xmlCreateMemoryParserCtxt(const char *buffer,
 						 int size);
