@@ -14,6 +14,7 @@
 
 #include <stdio.h>
 #include <libxml/xmlversion.h>
+#include <libxml/xmlstring.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -97,22 +98,6 @@ typedef enum {
 #endif
 } xmlElementType;
 
-/**
- * xmlChar:
- *
- * This is a basic byte in an UTF-8 encoded string.
- * It's unsigned allowing to pinpoint case where char * are assigned
- * to xmlChar * (possibly making serialization back impossible).
- */
-
-typedef unsigned char xmlChar;
-
-/**
- * BAD_CAST:
- *
- * Macro to cast a string to an xmlChar * when one know its safe.
- */
-#define BAD_CAST (xmlChar *)
 
 /**
  * xmlNotation:
@@ -520,7 +505,8 @@ struct _xmlDoc {
 /**
  * xmlChildrenNode:
  *
- * Macro for compatibility naming layer with libxml1.
+ * Macro for compatibility naming layer with libxml1. Maps
+ * to "children."
  */
 #ifndef xmlChildrenNode
 #define xmlChildrenNode children
@@ -529,7 +515,8 @@ struct _xmlDoc {
 /**
  * xmlRootNode:
  *
- * Macro for compatibility naming layer with libxml1.
+ * Macro for compatibility naming layer with libxml1. Maps 
+ * to "children".
  */
 #ifndef xmlRootNode
 #define xmlRootNode children
@@ -979,6 +966,11 @@ XMLPUBFUN void XMLCALL
 		xmlBufferWriteQuotedString(xmlBufferPtr buf,
 					 const xmlChar *string);
 
+XMLPUBFUN void xmlAttrSerializeTxtContent(xmlBufferPtr buf,
+					 xmlDocPtr doc,
+					 xmlAttrPtr attr,
+					 const xmlChar *string);
+
 /*
  * Namespace handling.
  */
