@@ -89,6 +89,9 @@
 #include <libxml/relaxng.h>
 #include <libxml/xmlschemas.h>
 #endif
+#ifdef LIBXML_PATTERN_ENABLED
+#include <libxml/pattern.h>
+#endif
 
 #ifndef XML_XML_DEFAULT_CATALOG
 #define XML_XML_DEFAULT_CATALOG "file:///etc/xml/catalog"
@@ -160,6 +163,10 @@ static int chkregister = 0;
 #ifdef LIBXML_SAX1_ENABLED
 static int sax1 = 0;
 #endif /* LIBXML_SAX1_ENABLED */
+#ifdef LIBXML_PATTERN_ENABLED
+static const char *pattern = NULL;
+static xmlPatternPtr patternc = NULL;
+#endif
 static int options = 0;
 
 /*
@@ -620,6 +627,13 @@ static void processNode(xmlTextReaderPtr reader) {
     else {
 	printf(" %s\n", value);
     }
+#ifdef LIBXML_PATTERN_ENABLED
+    if (patternc) {
+        if (xmlPatternMatch(patternc, xmlTextReaderCurrentNode(reader)) == 1) {
+	    printf("Node matches pattern %s\n", pattern);
+	}
+    }
+#endif
 }
 
 static void streamFile(char *filename) {
@@ -680,7 +694,11 @@ static void streamFile(char *filename) {
 	}
 	ret = xmlTextReaderRead(reader);
 	while (ret == 1) {
-	    if (debug)
+	    if ((debug)
+#ifdef LIBXML_PATTERN_ENABLED
+	        || (patternc)
+#endif
+	       )
 		processNode(reader);
 	    ret = xmlTextReaderRead(reader);
 	}
@@ -709,10 +727,10 @@ static void streamFile(char *filename) {
 #ifdef LIBXML_SCHEMAS_ENABLED
 	if (relaxng != NULL) {
 	    if (xmlTextReaderIsValid(reader) != 1) {
-		printf("%s fails to validate\n", filename);
+		fprintf(stderr, "%s fails to validate\n", filename);
 		progresult = 3;
 	    } else {
-		printf("%s validates\n", filename);
+		fprintf(stderr, "%s validates\n", filename);
 	    }
 	}
 #endif
@@ -721,7 +739,7 @@ static void streamFile(char *filename) {
 	 */
 	xmlFreeTextReader(reader);
 	if (ret != 0) {
-	    printf("%s : failed to parse\n", filename);
+	    fprintf(stderr, "%s : failed to parse\n", filename);
 	    progresult = 1;
 	}
     } else {
@@ -748,7 +766,11 @@ static void walkDoc(xmlDocPtr doc) {
 	}
 	ret = xmlTextReaderRead(reader);
 	while (ret == 1) {
-	    if (debug)
+	    if ((debug)
+#ifdef LIBXML_PATTERN_ENABLED
+	        || (patternc)
+#endif
+	       )
 		processNode(reader);
 	    ret = xmlTextReaderRead(reader);
 	}
@@ -757,7 +779,7 @@ static void walkDoc(xmlDocPtr doc) {
 	}
 	xmlFreeTextReader(reader);
 	if (ret != 0) {
-	    printf("failed to walk through the doc\n");
+	    fprintf(stderr, "failed to walk through the doc\n");
 	    progresult = 1;
 	}
     } else {
@@ -985,7 +1007,7 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	if ((timing) && (!repeat)) {
 	    startTimer();
 	}
-	xmlXIncludeProcess(doc);
+	xmlXIncludeProcessFlags(doc, options);
 	if ((timing) && (!repeat)) {
 	    endTimer("Xinclude processing");
 	}
@@ -1023,14 +1045,14 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	    if (node != NULL) {
 		nb = xmlValidGetValidElements(node->last, NULL, list, 256);
 		if (nb < 0) {
-		    printf("could not get valid list of elements\n");
+		    fprintf(stderr, "could not get valid list of elements\n");
 		} else if (nb == 0) {
-		    printf("No element can be inserted under root\n");
+		    fprintf(stderr, "No element can be inserted under root\n");
 		} else {
-		    printf("%d element types can be inserted under root:\n",
+		    fprintf(stderr, "%d element types can be inserted under root:\n",
 		           nb);
 		    for (i = 0;i < nb;i++) {
-			 printf("%s\n", (char *) list[i]);
+			 fprintf(stderr, "%s\n", (char *) list[i]);
 		    }
 		}
 	    }
@@ -1044,6 +1066,8 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 #endif /* LIBXML_READER_ENABLED */
 #ifdef LIBXML_OUTPUT_ENABLED
     if (noout == 0) {
+        int ret;
+
 	/*
 	 * print it.
 	 */
@@ -1122,14 +1146,25 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	    }
 	    else if (encoding != NULL) {
 	        if ( format ) {
-		    xmlSaveFormatFileEnc(output ? output : "-", doc, encoding, 1);
+		    ret = xmlSaveFormatFileEnc(output ? output : "-", doc,
+		                               encoding, 1);
 		}
 		else {
-		    xmlSaveFileEnc(output ? output : "-", doc, encoding);
+		    ret = xmlSaveFileEnc(output ? output : "-", doc, encoding);
+		}
+		if (ret < 0) {
+		    fprintf(stderr, "failed save to %s\n",
+		            output ? output : "-");
+		    progresult = 6;
 		}
 	    }
 	    else if (format) {
-		xmlSaveFormatFile(output ? output : "-", doc, 1);
+		ret = xmlSaveFormatFile(output ? output : "-", doc, 1);
+		if (ret < 0) {
+		    fprintf(stderr, "failed save to %s\n",
+		            output ? output : "-");
+		    progresult = 6;
+		}
 	    }
 	    else {
 		FILE *out;
@@ -1273,11 +1308,11 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 		stderr);
 	ret = xmlRelaxNGValidateDoc(ctxt, doc);
 	if (ret == 0) {
-	    printf("%s validates\n", filename);
+	    fprintf(stderr, "%s validates\n", filename);
 	} else if (ret > 0) {
-	    printf("%s fails to validate\n", filename);
+	    fprintf(stderr, "%s fails to validate\n", filename);
 	} else {
-	    printf("%s validation generated an internal error\n",
+	    fprintf(stderr, "%s validation generated an internal error\n",
 		   filename);
 	}
 	xmlRelaxNGFreeValidCtxt(ctxt);
@@ -1299,11 +1334,11 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 		stderr);
 	ret = xmlSchemaValidateDoc(ctxt, doc);
 	if (ret == 0) {
-	    printf("%s validates\n", filename);
+	    fprintf(stderr, "%s validates\n", filename);
 	} else if (ret > 0) {
-	    printf("%s fails to validate\n", filename);
+	    fprintf(stderr, "%s fails to validate\n", filename);
 	} else {
-	    printf("%s validation generated an internal error\n",
+	    fprintf(stderr, "%s validation generated an internal error\n",
 		   filename);
 	}
 	xmlSchemaFreeValidCtxt(ctxt);
@@ -1451,7 +1486,7 @@ static void usage(const char *name) {
 #ifdef LIBXML_CATALOG_ENABLED
     printf("\t--catalogs : use SGML catalogs from $SGML_CATALOG_FILES\n");
     printf("\t             otherwise XML Catalogs starting from \n");
-    printf("\t         " XML_XML_DEFAULT_CATALOG " are activated by default\n");
+    printf("\t         %s are activated by default\n", XML_XML_DEFAULT_CATALOG);
     printf("\t--nocatalogs: deactivate all catalogs\n");
 #endif
     printf("\t--auto : generate a small doc on the fly\n");
@@ -1464,6 +1499,9 @@ static void usage(const char *name) {
     printf("\t--stream : use the streaming interface to process very large files\n");
     printf("\t--walker : create a reader and walk though the resulting doc\n");
 #endif /* LIBXML_READER_ENABLED */
+#ifdef LIBXML_PATTERN_ENABLED
+    printf("\t--pattern pattern_value : test the pattern support\n");
+#endif
     printf("\t--chkregister : verify the node registration code\n");
 #ifdef LIBXML_SCHEMAS_ENABLED
     printf("\t--relaxng schema : do RelaxNG validation against the schema\n");
@@ -1586,16 +1624,19 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--postvalid"))) {
 	    postvalid++;
 	    loaddtd++;
+	    options |= XML_PARSE_DTDLOAD;
 	} else if ((!strcmp(argv[i], "-dtdvalid")) ||
 	         (!strcmp(argv[i], "--dtdvalid"))) {
 	    i++;
 	    dtdvalid = argv[i];
 	    loaddtd++;
+	    options |= XML_PARSE_DTDLOAD;
 	} else if ((!strcmp(argv[i], "-dtdvalidfpi")) ||
 	         (!strcmp(argv[i], "--dtdvalidfpi"))) {
 	    i++;
 	    dtdvalidfpi = argv[i];
 	    loaddtd++;
+	    options |= XML_PARSE_DTDLOAD;
         }
 #endif /* LIBXML_VALID_ENABLED */
 	else if ((!strcmp(argv[i], "-dropdtd")) ||
@@ -1732,6 +1773,12 @@ main(int argc, char **argv) {
         } else if ((!strcmp(argv[i], "-nonet")) ||
                    (!strcmp(argv[i], "--nonet"))) {
 	    options |= XML_PARSE_NONET;
+#ifdef LIBXML_PATTERN_ENABLED
+        } else if ((!strcmp(argv[i], "-pattern")) ||
+                   (!strcmp(argv[i], "--pattern"))) {
+	    i++;
+	    pattern = argv[i];
+#endif
 	} else {
 	    fprintf(stderr, "Unknown option %s\n", argv[i]);
 	    usage(argv[0]);
@@ -1847,7 +1894,18 @@ main(int argc, char **argv) {
 	    endTimer("Compiling the schemas");
 	}
     }
-#endif
+#endif /* LIBXML_SCHEMAS_ENABLED */
+#ifdef LIBXML_PATTERN_ENABLED
+    if (pattern != NULL) {
+        patternc = xmlPatterncompile((const xmlChar *) pattern, NULL, 0, NULL);
+	if (patternc == NULL) {
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Pattern %s failed to compile\n", pattern);
+            progresult = 7;
+	    pattern = NULL;
+	}
+    }
+#endif /* LIBXML_PATTERN_ENABLED */
     for (i = 1; i < argc ; i++) {
 	if ((!strcmp(argv[i], "-encode")) ||
 	         (!strcmp(argv[i], "--encode"))) {
@@ -1881,6 +1939,13 @@ main(int argc, char **argv) {
 	    i++;
 	    continue;
         }
+#ifdef LIBXML_PATTERN_ENABLED
+        if ((!strcmp(argv[i], "-pattern")) ||
+	    (!strcmp(argv[i], "--pattern"))) {
+	    i++;
+	    continue;
+	}
+#endif
 	if ((timing) && (repeat))
 	    startTimer();
 	/* Remember file names.  "-" means stdin.  <sven@zen.org> */
@@ -1931,6 +1996,10 @@ main(int argc, char **argv) {
     if (wxschemas != NULL)
 	xmlSchemaFree(wxschemas);
     xmlRelaxNGCleanupTypes();
+#endif
+#ifdef LIBXML_PATTERN_ENABLED
+    if (patternc != NULL)
+        xmlFreePattern(patternc);
 #endif
     xmlCleanupParser();
     xmlMemoryDump();
